<?php

namespace Drupal\wordfilter\Plugin\Filter;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\filter\Attribute\Filter;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Drupal\filter\Plugin\FilterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a filter for filtering specific words.
 */
#[Filter(
  id: "wordfilter",
  title: new TranslatableMarkup("Apply filtering of words"),
  type: FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE,
  description: new TranslatableMarkup("Filter out words by given Wordfilter configurations (choose below).")
)]
class WordFilter extends FilterBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs a WordFilter object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $themeManager
   *   The theme manager service.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user account proxy.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected ThemeManagerInterface $themeManager,
    protected AccountProxyInterface $currentUser,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('theme.manager'),
      $container->get('current_user'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Get a list of active Wordfilter configurations for this filter.
   *
   * @return array|null
   *   Array of word filter configuration.
   */
  public function activeWordfilterConfigs(): ?array {
    $activeConfigs = [];
    $settings_active = !empty($this->settings['active_wordfilter_configs']) ? $this->settings['active_wordfilter_configs'] : [];

    if (!empty($settings_active)) {
      try {
        $storage = $this->entityTypeManager->getStorage('wordfilter_configuration');
        $activeConfigs = $storage->loadMultiple($settings_active);
      }
      catch (InvalidPluginDefinitionException | PluginNotFoundException) {
        // @todo log something.
      }
    }

    return $activeConfigs;
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode): FilterProcessResult {
    $active_configs = $this->activeWordfilterConfigs();

    if (!empty($active_configs)) {
      foreach ($active_configs as $wordfilter_config) {
        $process = $wordfilter_config->getProcess();
        $text = $process->filterWords($text, $wordfilter_config, $langcode);
      }
    }

    return (new FilterProcessResult($text))->addCacheTags(['config:wordfilter_configuration_list']);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    try {
      $storage = $this->entityTypeManager->getStorage('wordfilter_configuration');
      $wordfilter_configs = $storage->loadMultiple();

      foreach ($wordfilter_configs as $key => $config) {
        $wordfilter_configs[$key] = $config->label();
      }

      $descriptions = [
        $this->t('Choose the available Wordfilter configurations as active filters.'),
      ];
      if ($this->currentUser->hasPermission('access wordfilter configurations page')) {
        $descriptions[] = $this->t('You can create and manage Wordfilter configurations at the <a target="_blank" href=":url">Wordfilter configuration page</a>.', [':url' => '/admin/config/wordfilter_configuration']);
      }
      $form['active_wordfilter_configs'] = [
        '#type' => 'select',
        '#title' => $this->t('Active Wordfilter configurations'),
        '#options' => $wordfilter_configs,
        '#default_value' => !empty($this->settings['active_wordfilter_configs']) ? $this->settings['active_wordfilter_configs'] : [],
        '#description' => $this->themeManager->render('item_list', ['items' => $descriptions]),
        '#multiple' => TRUE,
      ];
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException) {
    }

    return $form;
  }

}
