<?php

namespace Drupal\wordfilter;

use Drupal\wordfilter\Entity\WordfilterConfigurationInterface;

/**
 * The word filter item object.
 */
class WordfilterItem {

  /**
   * The parent entity, which is holding the filtering item.
   *
   * @var \Drupal\wordfilter\Entity\WordfilterConfigurationInterface
   */
  protected WordfilterConfigurationInterface $parent;

  /**
   * The item data as array.
   *
   * Contains the following keys:
   *   'delta': The delta of the filtering item.
   *   'substitute': The substitution string.
   *   'filter_words': The list of words to filter.
   *                   This list is being stored as a string,
   *                   words are being separated by ", ".
   *
   * @var array
   */
  protected array $data;

  /**
   * WordfilterItem constructor.
   *
   * @param \Drupal\wordfilter\Entity\WordfilterConfigurationInterface $parent
   *   The parent configuration.
   * @param array &$data
   *   The item values as array.
   *   Must contain the keys 'delta', 'substitute' and 'filter_words',
   *   which are all strings.
   */
  public function __construct(WordfilterConfigurationInterface $parent, array &$data) {
    $this->setParent($parent);
    $this->setData($data);
  }

  /**
   * Get the parent entity.
   *
   * @return \Drupal\wordfilter\Entity\WordfilterConfigurationInterface
   *   Configuration interface.
   */
  public function getParent(): WordfilterConfigurationInterface {
    return $this->parent;
  }

  /**
   * Set the parent entity.
   *
   * @param \Drupal\wordfilter\Entity\WordfilterConfigurationInterface $parent
   *   Parent configuration.
   */
  protected function setParent(WordfilterConfigurationInterface $parent): void {
    $this->parent = $parent;
  }

  /**
   * Set the item data.
   *
   * @param array &$data
   *   Data array.
   */
  protected function setData(array &$data): void {
    $this->data = &$data;
  }

  /**
   * Get the delta of this filtering item.
   *
   * @return int
   *   Delta value.
   */
  public function getDelta(): int {
    return $this->data['delta'];
  }

  /**
   * Set the delta of this filtering item.
   *
   * @param int $delta
   *   Delta value.
   */
  public function setDelta(int $delta): void {
    $this->data['delta'] = $delta;
  }

  /**
   * Get the substitution string.
   *
   * @return string
   *   Substitute string.
   */
  public function getSubstitute(): string {
    return $this->data['substitute'];
  }

  /**
   * Set the substitution string.
   *
   * @param string $substitute
   *   The substitution string.
   */
  public function setSubstitute(string $substitute): void {
    $this->data['substitute'] = $substitute;
  }

  /**
   * Get the filter words.
   *
   * @return array
   *   An array of the filter words.
   */
  public function getFilterWords(): array {
    return $this->wordsToArray($this->data['filter_words']);
  }

  /**
   * Set the filter words.
   *
   * @param mixed $words
   *   A string or array of filter words.
   *   As a string, multiple words are to be separated by ", ".
   */
  public function setFilterWords(mixed $words): void {
    if (is_string($words)) {
      $words = $this->wordsToArray($words);
    }
    // Filter out duplicates.
    // @todo Replace with a case-insensitive filter.
    $words = array_unique($words);

    $words = implode(", ", $words);
    $this->data['filter_words'] = $words;
  }

  /**
   * Helper function to transform a string of words to an array.
   *
   * @param string $words_string
   *   The string of words, usually separated by ", ".
   *
   * @return array
   *   The array of words.
   */
  protected function wordsToArray(string $words_string): array {
    $words_string = str_replace(["\n", "\r"], ',', $words_string);
    $words_array = !empty($words_string) ? explode(',', $words_string) : [];
    return array_filter(array_map('trim', $words_array), function ($v) {
      return $v !== '';
    });
  }

  /**
   * Array representation of this object.
   *
   * @return array
   *   Data to array.
   */
  public function toArray(): array {
    return $this->data;
  }

}
