<?php

namespace Drupal\Tests\wordfilter\Functional;

use Drupal\node\NodeInterface;
use Drupal\Tests\BrowserTestBase;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\user\UserInterface;

/**
 * Contains the appliance tests regarding Wordfilter configurations.
 *
 * @package Drupal\Tests\wordfilter\Functional
 * @group filter
 */
class WordfilterApplianceTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected $profile = 'standard';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['wordfilter', 'node', 'comment'];

  /**
   * The user.
   *
   * @var \Drupal\user\UserInterface|null
   */
  protected ?UserInterface $user = NULL;

  /**
   * The node.
   *
   * @var \Drupal\node\NodeInterface|null
   */
  protected ?NodeInterface $node = NULL;

  /**
   * Config ID.
   *
   * @var string
   */
  protected string $wordfilterConfigId = 'my_superduper_wordfilter_config';

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function setUp(): void {
    parent::setUp();

    $this->user = $this->drupalCreateUser([
      'access wordfilter configurations page',
      'administer wordfilter configurations',
      'administer filters',
      'administer nodes',
      'administer content types',
      'administer comments',
      'administer comment types',
    ]);

    $this->node = $this->nodeInstance();
  }

  /**
   * Tests the creation and editing of Wordfilter configurations.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testWordfilterConfigCreationEdit(): void {
    $this->drupalLogin($this->user);
    $this->createWordfilterConfig();

    // Add a few more items.
    $this->drupalGet('admin/config/wordfilter_configuration/' . $this->wordfilterConfigId . '/edit');
    $this->submitForm([], 'Add another item');
    $this->submitForm([], 'Add another item');
    $this->submitForm([
      'items[1][substitute]' => '---',
      'items[1][filter_words]' => 'Sit, Amet, Consectetur',
      'items[2][substitute]' => '...',
      'items[2][filter_words]' => 'Adipiscing, Elit, Vestibulum',
    ], 'Save');

    $this->drupalGet('admin/config/wordfilter_configuration/' . $this->wordfilterConfigId . '/edit');
    $this->assertSession()->fieldValueEquals('items[0][substitute]', '***');
    $this->assertSession()
      ->fieldValueEquals('items[0][filter_words]', 'Lorem, Ipsum, Dolor');
    $this->assertSession()->fieldValueEquals('items[1][substitute]', '---');
    $this->assertSession()
      ->fieldValueEquals('items[1][filter_words]', 'Sit, Amet, Consectetur');
    $this->assertSession()->fieldValueEquals('items[2][substitute]', '...');
    $this->assertSession()
      ->fieldValueEquals('items[2][filter_words]', 'Adipiscing, Elit, Vestibulum');

    // Remove an item.
    $this->submitForm([], 'edit-items-1-remove');
    $this->submitForm([], 'Save');

    $this->drupalGet('admin/config/wordfilter_configuration/' . $this->wordfilterConfigId . '/edit');
    $this->assertSession()->fieldValueEquals('items[0][substitute]', '***');
    $this->assertSession()
      ->fieldValueEquals('items[0][filter_words]', 'Lorem, Ipsum, Dolor');
    $this->assertSession()->fieldNotExists('items[1][substitute]');
    $this->assertSession()->fieldNotExists('items[1][filter_words]');
    $this->assertSession()->fieldValueEquals('items[2][substitute]', '...');
    $this->assertSession()
      ->fieldValueEquals('items[2][filter_words]', 'Adipiscing, Elit, Vestibulum');
  }

  /**
   * Tests the appliance of Wordfilter configurations on text formats.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testApplianceOnTextFormat(): void {
    $this->drupalLogin($this->user);
    $this->createWordfilterConfig();
    $this->addWordfilterConfigToBasicHtmlFormat();

    $this->drupalGet('node/' . $this->nodeInstance()->id());
    $this->assertSession()->pageTextContains("TITLE The Lorem Ipsum Node");
    $this->assertSession()->pageTextContains("BODY *** *** *** Sit Amet");
  }

  /**
   * Tests the appliance of Wordfilter configurations on nodes.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testApplianceOnNode(): void {
    $this->drupalLogin($this->user);
    $this->createWordfilterConfig();
    $this->addWordfilterConfigToArticle();

    $this->drupalGet('node/' . $this->nodeInstance()->id());
    $this->assertSession()->pageTextContains("TITLE The *** *** Node");
    $this->assertSession()->pageTextContains("BODY *** *** *** Sit Amet");
  }

  /**
   * Tests the appliance of Wordfilter configurations on comments.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testApplianceOnComment(): void {
    $this->drupalLogin($this->user);
    $this->createWordfilterConfig();
    $this->addWordfilterConfigToComments();
    $this->addAndViewCommentOnNode();

    $this->drupalGet('node/' . $this->nodeInstance()->id());
    $this->assertSession()
      ->pageTextContains("COMMENT-SUBJECT The *** *** Comment");
    $this->assertSession()
      ->pageTextContains("COMMENT-BODY *** *** *** Sit Amet");
  }

  /**
   * Helper function to create a Wordfilter configuration.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function createWordfilterConfig(): void {
    $this->drupalGet('admin/config/wordfilter_configuration/add');
    $this->assertSession()->statusCodeEquals(200);

    $this->submitForm([
      'id' => $this->wordfilterConfigId,
      'label' => 'My superduper Wordfilter config',
      'process_id' => 'default',
      'items[0][substitute]' => '***',
      'items[0][filter_words]' => 'Lorem, Ipsum, Dolor',
    ], 'Save');

    $this->assertSession()->responseContains(
      new FormattableMarkup('Created the %label Wordfilter configuration.',
        ['%label' => 'My superduper Wordfilter config']));
  }

  /**
   * Helper function to add the Wordfilter configuration to text format.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function addWordfilterConfigToBasicHtmlFormat(): void {
    $this->drupalGet('admin/config/content/formats/manage/basic_html');
    $this->assertSession()->statusCodeEquals(200);

    $this->submitForm([
      'filters[wordfilter][status]' => TRUE,
      'filters[wordfilter][settings][active_wordfilter_configs][]' => [$this->wordfilterConfigId],
    ], 'Save configuration');

    $this->assertSession()->responseContains(
      new FormattableMarkup('The text format %format has been updated.',
        ['%format' => 'Basic HTML']));
  }

  /**
   * Helper function to add the Wordfilter configuration to content types.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function addWordfilterConfigToArticle(): void {
    $this->drupalGet('/admin/structure/types/manage/article');
    $this->assertSession()->statusCodeEquals(200);

    $this->submitForm([
      'active_wordfilter_configs[]' => [$this->wordfilterConfigId],
    ], 'Save');

    $this->assertSession()->responseContains(
      new FormattableMarkup('The content type %type has been updated.',
        ['%type' => 'Article']));
  }

  /**
   * Helper function to add the Wordfilter configuration to comments.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function addWordfilterConfigToComments(): void {
    $this->drupalGet('/admin/structure/comment/manage/comment');
    $this->assertSession()->statusCodeEquals(200);

    $this->submitForm([
      'active_wordfilter_configs[]' => [$this->wordfilterConfigId],
    ], 'Save');

    $this->assertSession()->responseContains(
      new FormattableMarkup('Comment type %type has been updated.',
        ['%type' => 'Default comments']));
  }

  /**
   * Helper function to add and view a comment on the given node instance.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   */
  protected function addAndViewCommentOnNode(): void {
    $this->drupalGet('node/' . $this->nodeInstance()->id());

    $form = [
      'subject[0][value]' => 'COMMENT-SUBJECT The Lorem Ipsum Comment',
      'comment_body[0][value]' => 'COMMENT-BODY Lorem Ipsum Dolor Sit Amet',
    ];

    $this->submitForm($form, 'Preview', 'comment-form');
    $this->assertSession()
      ->pageTextContains("COMMENT-SUBJECT The *** *** Comment");
    $this->assertSession()
      ->pageTextContains("COMMENT-BODY *** *** *** Sit Amet");

    $this->submitForm($form, 'Save', 'comment-form--2');
    $this->assertSession()
      ->pageTextContains("COMMENT-SUBJECT The *** *** Comment");
    $this->assertSession()
      ->pageTextContains("COMMENT-BODY *** *** *** Sit Amet");
  }

  /**
   * Create and get a node instance for all appliance tests.
   *
   * @return \Drupal\node\NodeInterface
   *   Node.
   */
  protected function nodeInstance(): NodeInterface {
    if (!isset($this->node)) {
      $this->node = $this->createNode([
        'type' => 'article',
        'title' => 'TITLE The Lorem Ipsum Node',
        'status' => 1,
        'body' => [
          [
            'value' => 'BODY Lorem Ipsum Dolor Sit Amet',
            'format' => 'basic_html',
          ],
        ],
      ]);
    }

    return $this->node;
  }

}
